<?php

namespace UBRIR;

abstract class CoreSmartLogger
{
    protected $useLogs = true;
    protected $logDir;
    protected $logFile;

    public function __construct($ubrrCoreDir)
    {
        $this->logDir = $ubrrCoreDir . 'logs/';
        $this->logFile = $this->logDir . 'ubrr.log';

        //TODO: каждый чих вызова только в режиме отладки (временный файл для определения нужно в основной лог переносить или нет)
        //TODO: дробление по размеру файла (удаление очень старых логов)
        //TODO: вывод из лога сделать без учета просто вызывов. Можно вызовы из мест отдельным списком форматировать, при это пустые можно скрывать при отображении. В общем модернизация вывода лога
        $this->StartLog();
    }

    private function StartLog()
    {
        $SearchedClasses = Array(
            'UBRIR\GatewayPayment',
            'UBRIR\GatewayForAdmin',
            'UBRIR\NewCertificates',
            'UBRIR\Diagnostics'
        );

        $backtrace = debug_backtrace(false, 7);

        $isAdminFunction = false;

        for ($i = 0; $i < count($backtrace); $i++) {
            if ($backtrace[$i]['function'] === 'GetGatewayPanel') {
                $isAdminFunction = true;
                break;
            }
        }

        if ($isAdminFunction) {
            if ($backtrace[$i + 1]['function'] == '__call') {
                //MagicMethods
                $callerRecord = $backtrace[$i + 2];

                $functionName = isset($callerRecord['args'][0]) ? $callerRecord['args'][0] : 'AdminMagic';
            } else {
                $callerRecord = $backtrace[$i + 1];
                $functionName = $callerRecord['function'];
            }

            $callerMethodName = $functionName . ' in ' . $callerRecord['class'];
        } else {
            $caller = Array('file' => '', 'line' => '', 'class' => '', 'function' => '');

            for ($i = 0; $i < count($backtrace); $i++) {
                if (isset($backtrace[$i]['class']) &&
                    in_array($backtrace[$i]['class'], $SearchedClasses) &&
                    $backtrace[$i]['function'] === '__construct') {

                    if (isset($backtrace[$i + 1]['class']))
                        $caller = $backtrace[$i + 1];
                    else {
                        $caller = $backtrace[$i];
                    }
                    break;
                }
            }
            $callerMethodName = $caller['class'] . '->' . $caller['function'] . ' [' . $caller['file'] . ':' . $caller['line'] . ']';
        }

        $data = $callerMethodName . ' ' . date('d.m.Y H:i:s');
        $this->Log('[START]' . $data);
    }

    public function __destruct()
    {
        $this->Log('[END]');
    }

    protected function EnabledLog()
    {
        $this->useLogs = true;
        return $this;
    }

    protected function DisableLog()
    {
        $this->useLogs = false;
        return $this;
    }

    public function LogError($errorData)
    {
        if (!is_string($errorData))
            $data = print_r($errorData, true);
        else
            $data = $errorData;


        return '\tERROR' . $data;
    }

    public function Log($data)
    {
        if (!$this->useLogs) return $this;

        if (!is_string($data))
            $data = print_r($data, true);

        file_put_contents($this->logFile, $data . "\r\n", FILE_APPEND);

        return $this;
    }

    public function LogErrors(DataContainer $response)
    {
        if (!$this->useLogs) return $this;

        $arMessages = $response->getMessages();
        if (is_array($arMessages) && count($arMessages) > 0)
            foreach ($arMessages as $key => $list) {
                foreach ($list as $message) {
                    file_put_contents($this->logFile, "\t" . $key . ' ' . $message . "\r\n", FILE_APPEND);
                }
            }

        return $this;
    }

    /**
     * Получаем последние записи из лога, разделяем их по началу лога [START] и выводим в порядке: сначала последние
     * @return array
     */
    public function GetDebugLogRecords()
    {
        if (file_exists($this->logFile)) {
            $file_size = filesize($this->logFile);
            $bytes = 170000;
            if ($file_size > $bytes) {
                $cut = $file_size - $bytes;
                $handle = fopen($this->logFile, "r");
                fseek($handle, $cut);
                $logString = fread($handle, $bytes);
            } else {
                $logString = file_get_contents($this->logFile);
            }
            //$logString = str_replace('[END]', '<br>Конец сеанса', $logString);

            $arLogRecords = explode('[START]', $logString);
            $arReversedLog = array_reverse($arLogRecords);
            array_pop($arReversedLog);
            return $arReversedLog;
        } else {
            return Array();
        }
    }

    public function GetLogFileUrl()
    {
        $logUrl = str_replace($_SERVER['DOCUMENT_ROOT'], '', $this->logFile);

        if ($logUrl[0] !== '/') $logUrl = '/' . $logUrl;

        $logUrl = 'http://' . $_SERVER['HTTP_HOST'] . $logUrl;

        return $logUrl;
    }

    protected function FormatLogErrors($logRecords)
    {
        foreach ($logRecords as $key => $record) {
            $timeLineEnd = strpos($record, "\r\n") + strlen("\r\n");
            $timeLine = substr($record, 0, $timeLineEnd);

            $errorsBlock = '';
            $firstErrorLineStart = strpos($record, "\tERROR");
            if ($firstErrorLineStart !== false) {
                $lastErrorLineStart = strrpos($record, "\tERROR");

                $lastErrorLineEnd = strpos($record, "\r\n", $lastErrorLineStart);

                $errorsBlock = substr($record, $firstErrorLineStart, ($lastErrorLineEnd - $firstErrorLineStart) + strlen("\r\n"));
                $errorsBlock = str_replace(Array("\tERROR", "\r\n"), Array('<span class="errorLine">ERROR', "</span>\r\n"), $errorsBlock);

                $recordText = substr($record, $timeLineEnd, $firstErrorLineStart - $timeLineEnd) . substr($record, $lastErrorLineEnd);
            } else {
                $recordText = substr($record, $timeLineEnd);
            }

            $haveEndTag = false;
            if (strrpos($recordText, '[END]') !== false) {
                $haveEndTag = true;
                $recordText = str_replace('[END]', '', $recordText);
            }

            $logRecords[$key] = array('start_time' => $timeLine, 'text' => $recordText, 'errors' => $errorsBlock, 'success_end' => $haveEndTag);
        }

        return $logRecords;
    }

    public function GetLog()
    {
        $responseData = new DataContainer();
        $logRecords = $this->GetDebugLogRecords();

        $logRecords = $this->FormatLogErrors($logRecords);
        $responseData->SetResultRecord('log_records', $logRecords);
        $responseData->setSuccess();

        return $responseData;
    }

    public function CleanLog()
    {
        if (file_exists($this->logFile)) {
            return unlink($this->logFile);
        }
        return false;
    }

}