<?php

namespace UBRIR;

class DataContainer
{
    protected $success;
    public $arMessages;
    protected $rawXML;
    private $XMLFields;
    private $XMLFields_KeyCaseLower;
    private $parsedResponse;
    public $XMLNestedFields;

    private $ResponseFields;

    private $_arResult;

    const _defaultResultLabel = 'Ошибка при запросе';

    function __construct($success = false, $arMessages = Array())
    {
        $this->success = $success;

        if (is_array($arMessages)) {
            foreach ($arMessages as $code => $message) {
                $this->AddMessage($message, $code);
            }
        }

        $this->_arResult = Array(
            'label' => self::_defaultResultLabel,
            'html' => '',
        );

        return $this;
    }

    public function SetResultRecord($key, $value)
    {
        $this->_arResult[$key] = $value;

        return $this;
    }

    public function CopyFieldsToResultArray()
    {
        $this->_arResult = array_merge($this->parsedResponse, $this->_arResult);
    }

    public function GetResultArray()
    {
        $autoSetPart = array('success' => intval($this->isSuccess()));

        if (!$this->isSuccess()) {
            if (!empty($this->_arResult['html'])) {
                $this->_arResult['html'] .= "<br/>";
            }

            $this->_arResult['html'] .= '<b>В процессе обработки возникли ошибки</b>: '."<br/>";
            $this->_arResult['html'] .= implode('<br/>', $this->GetErrors());
        }

        return array_merge($autoSetPart, $this->_arResult);
    }

    public function GetResponseStatus()
    {
        return strval($this->XMLNestedFields->Response->Status[0]);
    }

    public function SetParsedResponse($parsedResponse, $code = 'response')
    {
        $this->parsedResponse[strtolower($code)] = $parsedResponse;

        return $this;
    }

    public function GetParsedResponse()
    {
        return $this->parsedResponse;
    }

    public function AddMessage($message, $code)
    {
        if (!is_array($this->arMessages[$code])) {
            $this->arMessages[$code] = Array();
        }

        $this->arMessages[$code][] = $message;

        return $this;
    }

    public function AddError($message, $errorCode = '')
    {
        $this->success = false;

        if (!isset($this->arMessages['ERROR'])) {
            $this->arMessages['ERROR'] = Array();
        }

        if (!empty($errorCode)) {
            $message = '['.$errorCode.']: '.$message;
        }

        $this->arMessages['ERROR'][] = $message;

        return $this;
    }

    public function GetLastError()
    {
        if (!$this->isSuccess()) {
            $lastError = array_pop($this->arMessages['ERROR']);
            $this->arMessages['ERROR'][] = $lastError;

            return $lastError;
        }

        return false;
    }

    public function setSuccess()
    {
        $this->success = true;

        if ($this->_arResult['label'] == self::_defaultResultLabel) {
            $this->SetResultRecord('label', '');
        }

        return $this;
    }

    public function isSuccess()
    {
        return $this->success;
    }

    public function getRawXML()
    {
        return $this->rawXML;
    }

    public function GetMessages()
    {
        return $this->arMessages;
    }

    public function GetErrors()
    {
        return $this->arMessages['ERROR'];
    }

    public function SaveXMLData($rawXML, $nestedXMLFields, $arXMLFields)
    {
        $this->rawXML = $rawXML;
        $this->XMLNestedFields = $nestedXMLFields;
        $this->XMLFields = $arXMLFields;
        $this->XMLFields_KeyCaseLower = array_change_key_case($arXMLFields, CASE_LOWER);

        return $this;
    }

    public function addFieldValue($code, $value)
    {
        $this->ResponseFields[$code] = $value;
    }

    public function getFieldValue($code)
    {
        $value = false;

        $lCode = strtolower($code);
        if (isset($this->XMLFields[$code]) || isset($this->XMLFields_KeyCaseLower[$lCode])) {
            if (isset($this->XMLFields[$code])) {
                $value = $this->XMLFields[$code];
            } else {
                $value = $this->XMLFields_KeyCaseLower[$lCode];
            }
        } elseif (isset($this->ResponseFields[$code])) {
            $value = $this->ResponseFields[$code];
        }

        return $value;
    }

    //Помощники для выдерания полей
    public function GetPaymentUrl($TWPG_OrderId, $SessionId)
    {
        $url = $this->getFieldValue('Url');
        if ($url) {
            $url .= '?orderid='.$TWPG_OrderId.'&sessionid='.$SessionId;
        }

        return $url;
    }

    public function GetTWPG_OrderId()
    {
        return $this->getFieldValue('OrderId');
    }

    public function GetSessionId()
    {
        return $this->getFieldValue('SessionId');
    }
}