<?php

namespace UBRIR;

class GatewayPayment
{
    public function __construct(InterfaceConnectionConfig $connector = null, $useLogs = true)
    {
        if ($connector !== null) {
            Core::GetCore()->PlugInCMSConnector($connector);
        }
    }

    //Заполнение необходимых параметров и затем попытка передачи
    public function SetupAndPrepareToPay($orderId, $orderAmount, $approveCallbackURL, $cancelCallbackURL = '', $declineCallbackURL = '')
    {
        if (empty($cancelCallbackURL))
            $cancelCallbackURL = $approveCallbackURL;

        if (empty($declineCallbackURL))
            $declineCallbackURL = $approveCallbackURL;

        //умножаем на 100, потому что передача в платежный шлюз в копейках
        $orderAmount = $orderAmount * 100;

        $serverResponse = ServerConnector::PrepareToPay($orderId, $orderAmount, $approveCallbackURL, $cancelCallbackURL, $declineCallbackURL);

        return $serverResponse;
    }

    public function SaveTWPGOrder($orderId, $TWPG_OrderId, $SessionId, $paymentId = false)
    {
        $connector = Core::GetCore()->GetCMSConnector();

        return $connector->SaveTWPGOrder($orderId, $TWPG_OrderId, $SessionId, $paymentId);
    }

    public function SetupAndCheckOrderStatus($TWPG_OrderId, $TWPG_SessionId)
    {
        $serverResponse = ServerConnector::GetOrderStatus($TWPG_OrderId, $TWPG_SessionId);

        return $serverResponse;
    }

    //region Работа с сертификатами
    protected function ConvertCertificateToPEM($cerFile)
    {
        $certificateFilePath = substr($cerFile, 0, strrpos($cerFile, '.'));
        $certificatePEMFile = $certificateFilePath . '.pem';

        if (!file_exists($certificatePEMFile)) {
            if (file_exists($cerFile)) {
                $certificateCAcerContent = file_get_contents($cerFile);

                /* Convert .cer to .pem, cURL uses .pem */
                $certificateCApemContent = '-----BEGIN CERTIFICATE-----' . PHP_EOL
                    . chunk_split(base64_encode($certificateCAcerContent), 64, PHP_EOL)
                    . '-----END CERTIFICATE-----' . PHP_EOL;

                file_put_contents($certificatePEMFile, $certificateCApemContent);
            }
        }

        if (file_exists($certificatePEMFile)) {
            return $certificatePEMFile;
        }
        return false;
    }

    //endregion

    public function ShowPaymentError($continueLink, $errorMessage = '', PaymentResultViewer $resultViewer = null)
    {
        $core = Core::GetCore();

        $connector = $core->GetCMSConnector();

        if (empty($continueLink))
            $continueLink = $connector->GetDefaultContinueLink();

        if ($resultViewer == null)
            $resultViewer = new PaymentResultViewer();

        if (!empty($errorMessage))
            $resultViewer->SetFailureMessage($errorMessage);

        $view_data = $resultViewer
            ->SetContinueLink($continueLink)
            ->ViewData();

        echo $core->IncludeTemplate('view.paymentstatus', $view_data);
    }

    /**
     * Сохраняем данные о заказе и показываем страницу статуса оплаты
     * @param $continueLink
     * @param string $message
     * @param PaymentResultViewer|null $resultData
     */
    public function ShowPaymentSuccess($continueLink, $message = '', PaymentResultViewer $resultData = null)
    {
        $core = Core::GetCore();

        $connector = $core->GetCMSConnector();

        if (empty($continueLink))
            $continueLink = $connector->GetDefaultContinueLink();

        if ($resultData == null)
            $resultData = new PaymentResultViewer();

        $view_data = $resultData
            ->SetSuccessMessage($message)
            ->SetContinueLink($continueLink)
            ->ViewData();

        echo $core->IncludeTemplate('view.paymentstatus', $view_data);
    }
}