$(document).ready(function () {
    Admin_UBRR.Init();
});

(function () {
    var Admin_UBRR = {};
    window['Admin_UBRR'] = Admin_UBRR;

    var OpencartSubPages = {};
    Admin_UBRR['OpencartSubPages'] = OpencartSubPages;

    Admin_UBRR['Init'] = function () {
        OpencartSubPages.Init();
        ActivateSaveAllForms();
    };

    var $forms;
    var formDataSubmitted = false;
    var d = {
        'content': null,
        'tab_container': null,
        'tabs_ajax_places': {},
        'get_file_iframe': null
    };
    var ajaxTabLock = {};
    var lastOpenTabId = '';
    var checkTabsTimer = 0;
    var checkInterval = 250;


    function ActivateSaveAllForms() {
        $forms = $('#form-pp-std-uk, #form-pp-std-uk2');
        $forms.on('submit', CombineFormsDataAndSend);
    }

    function CombineFormsDataAndSend(e) {
        e.preventDefault();
        if (!formDataSubmitted) {
            formDataSubmitted = true;

            var form = e.target;

            var formData = new FormData(form);

            $forms.each(function () {
                var addForm = $(this).get(0);
                if (addForm !== form) {
                    var additionalParams = $(this).serializeArray();

                    for (var i = 0; i < additionalParams.length; i++) {
                        formData.set(additionalParams[i].name, additionalParams[i].value);
                    }
                }
            });

            var xhr = new XMLHttpRequest();
            xhr.open("POST", form.getAttribute('action') ? form.getAttribute('action') : window.location.href);
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4) {
                    if (xhr.status === 200) {
                        if (confirm('Настройки сохранены. Вернутся к списку платежных систем?')) {
                            window.location.href = xhr.responseURL;
                        }
                    }
                    else {
                        alert('Автообновление не удалось. Перезагрузите страницу вручную!');
                    }
                }
            };
            xhr.send(formData);
        }
    }

    Admin_UBRR['Ajax'] = function (request_type, ajax_data, callback, error_callback) {
        var jData = {};
        if (typeof(ajax_data) === 'object') {
            jData = ajax_data;
        }
        jData['ajax_type'] = request_type;

        jQuery.ajax({
            url: location.href,
            type: 'GET',
            data: jData,
            success: function (r) {
                if (typeof (callback) === 'function') {
                    callback(r);

                    OpencartSubPages.UpdateControlEvents();
                }
            },
            error: function (r) {
                if (typeof(error_callback) === 'function') {
                    error_callback(r);
                }
            }
        })
    };

    Admin_UBRR['FormAjaxPost'] = function (request_type, tab_id, form, callback) {
        var formData = new FormData(form);
        formData.append('ajax_type', request_type);
        formData.append('tab_id', tab_id);

        OpencartSubPages.setTabBusyMessage(tab_id);

        var xhr = new XMLHttpRequest();
        xhr.open("POST", form.getAttribute('action') ? form.getAttribute('action') : window.location.href);

        xhr.onreadystatechange = function () {
            if (xhr.readyState === 4) {
                var response = xhr.responseText;
                if (typeof(callback) === 'function') {
                    callback(response);

                    OpencartSubPages.UpdateControlEvents();
                }
            }
        };

        xhr.send(formData);
    };

    function clearLogClick() {
        Admin_UBRR.Ajax('clear_log', {}, function (r) {
            if (!ajaxTabLock['#tab-log'])
                d.tabs_ajax_places['#tab-log'].html(r);
        });
    }

    OpencartSubPages['Init'] = function () {
        d.content = $("#content");
        d.tab_container = d.content.find('.tab-content');

        OpencartSubPages.UpdateControlEvents();

        var $tabs = d.tab_container.find('.tab-pane');
        $tabs.each(function () {
            var tabId = '#' + $(this).attr('id');
            var $ajaxPlace = $(this).find('.ajaxPlace');
            //вкладки без ajaxPlace не подразумевают обновления по ajax
            if ($ajaxPlace.length === 0) {
                ajaxTabLock[tabId] = true;
            }
            else {
                ajaxTabLock[tabId] = false;

                d.tabs_ajax_places[tabId] = $ajaxPlace;
            }
        });

        lastOpenTabId = '#' + d.tab_container.find('.tab-pane.active').attr('id');

        checkTabsTimer = setInterval(checkWitchTabOpen, checkInterval);
    };

    function checkWitchTabOpen() {
        var currentActiveTabId = '#' + d.tab_container.find('.tab-pane.active').attr('id');
        if (currentActiveTabId !== lastOpenTabId) {
            lastOpenTabId = currentActiveTabId;
            if (!ajaxTabLock[lastOpenTabId]) {
                refreshTabContent(lastOpenTabId);
            }
        }
    }

    OpencartSubPages['setTabBusyMessage'] = function (tabId) {
        if (typeof(tabId) !== 'string')
            return false;

        if (tabId[0] !== '#')
            tabId = '#' + tabId;

        d.tabs_ajax_places[tabId].html('Загружаем страницу...');
    };

    function refreshTabContent(tabId) {
        if (!ajaxTabLock[tabId]) {
            ajaxTabLock[tabId] = true;

            var data = {'tab_id': tabId};

            OpencartSubPages.setTabBusyMessage(tabId);
            Admin_UBRR.Ajax('tab_refresh', data, function (response) {
                d.tabs_ajax_places[tabId].html(response);

                ajaxTabLock[tabId] = false;
            });
        }
    }

    function openTabClick() {
        var tabId = $(this).attr('data-tab-id');
        if ($(this).get(0).hasAttribute('data-tab-force')) {
            ajaxTabLock[tabId] = false;
            lastOpenTabId = '';
        }
        OpencartSubPages.OpenTab(tabId);
    }

    function runAjaxActionClick() {
        var actionName = $(this).attr('data-action-name');
        var actionType = $(this).attr('data-action-type') || 'ajax';

        switch (actionType) {
            case 'get_file': {
                if(d.get_file_iframe === null)
                {
                    var $fileIframe = $('<iframe id="file_download" style="display:none;"></iframe>');
                    d.get_file_iframe = $fileIframe.get(0);

                    var body = document.querySelector('body');
                    body.appendChild(d.get_file_iframe);
                }

                var src = location.href + '&ajax_type=ajax_action&action_name=' + actionName;
                d.get_file_iframe.src = src;
            }
                break;
            default: {
                var data = {'action_name': actionName};

                Admin_UBRR.Ajax('ajax_action', data, function (response) {

                    console.log(response);
                });
            }
        }
    }

    function AjaxFormSubmitEventHandler(e) {
        e.preventDefault();

        var tabId = '#' + $(this).closest('.tab-pane').attr('id');

        var formName = $(this).attr('name');

        ajaxTabLock[tabId] = true;
        var form = $(this).get(0);

        Admin_UBRR.FormAjaxPost('tab_refresh', tabId, form, function (response) {
            d.tabs_ajax_places[tabId].html(response);

            OpencartSubPages.UpdateControlEvents();
            ajaxTabLock[tabId] = false;
        });

        return false;
    };

    OpencartSubPages['UpdateControlEvents'] = function () {
        d.content.find('a.openTabLink').off('click', openTabClick);
        d.content.find('a.openTabLink').on('click', openTabClick);

        d.content.find('a.runAjaxAction').off('click', runAjaxActionClick);
        d.content.find('a.runAjaxAction').on('click', runAjaxActionClick);

        d.content.find('.button.deleteLog').off('click', clearLogClick);
        d.content.find('.button.deleteLog').on('click', clearLogClick);

        d.content.find('.tab-pane form:not(".disableAjax")').off('submit', AjaxFormSubmitEventHandler);
        d.content.find('.tab-pane form:not(".disableAjax")').on('submit', AjaxFormSubmitEventHandler);
    };

    OpencartSubPages['OpenTab'] = function (tabId) {
        var $tabSelector = d.content.find('.nav-tabs li a[href=' + tabId + ']');
        if ($tabSelector.length > 0) {
            $tabSelector.parent('li').addClass('active').siblings('li.active').removeClass('active');
            $(tabId).addClass('active').siblings('.tab-pane.active').removeClass('active');
        } else {
            alert('Неверный код вкладки: ' + tabId);
        }

        clearInterval(checkTabsTimer);
        checkWitchTabOpen();
        checkTabsTimer = setInterval(checkWitchTabOpen, checkInterval);
    };
})();